@@ -6,7 +6,7 @@ use HTTP::Headers::Util qw(split_header_words join_header_words);
 use LWP::Debug ();
 
 use vars qw($VERSION $EPOCH_OFFSET);
-$VERSION = sprintf("%d.%02d", q$Revision: 1.37 $ =~ /(\d+)\.(\d+)/);
+$VERSION = "5.810";
 
 # Legacy: because "use "HTTP::Cookies" used be the ONLY way
 #  to load the class HTTP::Cookies::Netscape.
@@ -199,7 +199,7 @@ sub extract_cookies
 
     if (@ns_set) {
 	# The old Netscape cookie format for Set-Cookie
-        # http://www.netscape.com/newsref/std/cookie_spec.html
+	# http://wp.netscape.com/newsref/std/cookie_spec.html
 	# can for instance contain an unquoted "," in the expires
 	# field, so we have to use this ad-hoc parser.
 	my $now = time();
@@ -217,6 +217,7 @@ sub extract_cookies
 	    my @cur;
 	    my $param;
 	    my $expires;
+	    my $first_param = 1;
 	    for $param (split(/;\s*/, $set)) {
 		my($k,$v) = split(/\s*=\s*/, $param, 2);
 		if (defined $v) {
@@ -227,8 +228,7 @@ sub extract_cookies
 		    $k =~ s/\s+$//;
 		    #print "$k => undef";
 		}
-		my $lc = lc($k);
-		if ($lc eq "expires") {
+		if (!$first_param && lc($k) eq "expires") {
 		    my $etime = str2time($v);
 		    if ($etime) {
 			push(@cur, "Max-Age" => str2time($v) - $now);
@@ -238,7 +238,9 @@ sub extract_cookies
 		else {
 		    push(@cur, $k => $v);
 		}
+		$first_param = 0;
 	    }
+            next unless @cur;
 	    next if $in_set2{$cur[0]};
 
 #	    push(@cur, "Port" => $req_port);
@@ -625,7 +627,7 @@ knows about.
 Cookies are a general mechanism which server side connections can use
 to both store and retrieve information on the client side of the
 connection.  For more information about cookies refer to
-<URL:http://www.netscape.com/newsref/std/cookie_spec.html> and
+<URL:http://wp.netscape.com/newsref/std/cookie_spec.html> and
 <URL:http://www.cookiecentral.com/>.  This module also implements the
 new style cookies described in I<RFC 2965>.
 The two variants of cookies are supposed to be able to coexist happily.
@@ -689,7 +691,7 @@ various other attributes like "Comment" and "CommentURL".
 This method file saves the state of the $cookie_jar to a file.
 The state can then be restored later using the load() method.  If a
 filename is not specified we will use the name specified during
-construction.  If the attribute I<ignore_discared> is set, then we
+construction.  If the attribute I<ignore_discard> is set, then we
 will even save cookies that are marked to be discarded.
 
 The default is to save a sequence of "Set-Cookie3" lines.
