@@ -2,7 +2,7 @@ package HTTP::Message;
 
 use strict;
 use vars qw($VERSION $AUTOLOAD);
-$VERSION = "5.810";
+$VERSION = "5.811";
 
 require HTTP::Headers;
 require Carp;
@@ -11,7 +11,14 @@ my $CRLF = "\015\012";   # "\r\n" is not portable
 $HTTP::URI_CLASS ||= $ENV{PERL_HTTP_URI_CLASS} || "URI";
 eval "require $HTTP::URI_CLASS"; die $@ if $@;
 
-*_is_utf8 = defined &utf8::is_utf8 ? \&utf8::is_utf8 : sub { 0 };
+*_utf8_downgrade = defined(&utf8::downgrade) ?
+    sub {
+        utf8::downgrade($_[0], 1) or
+            Carp::croak("HTTP::Message content must be bytes")
+    }
+    :
+    sub {
+    };
 
 sub new
 {
@@ -29,9 +36,7 @@ sub new
 	$header = HTTP::Headers->new;
     }
     if (defined $content) {
-        if (_is_utf8($content)) {
-            Carp::croak("HTTP::Message content not bytes");
-        }
+        _utf8_downgrade($content);
     }
     else {
         $content = '';
@@ -110,9 +115,7 @@ sub content  {
 
 sub _set_content {
     my $self = $_[0];
-    if (_is_utf8($_[1])) {
-        Carp::croak("HTTP::Message content not bytes")
-    }
+    _utf8_downgrade($_[1]);
     if (!ref($_[1]) && ref($self->{_content}) eq "SCALAR") {
 	${$self->{_content}} = $_[1];
     }
@@ -132,9 +135,7 @@ sub add_content
     my $chunkref = \$_[0];
     $chunkref = $$chunkref if ref($$chunkref);  # legacy
 
-    if (_is_utf8($$chunkref)) {
-        Carp::croak("HTTP::Message added content not bytes");
-    }
+    _utf8_downgrade($$chunkref);
 
     my $ref = ref($self->{_content});
     if (!$ref) {
@@ -149,6 +150,12 @@ sub add_content
     delete $self->{_parts};
 }
 
+sub add_content_utf8 {
+    my($self, $buf)  = @_;
+    utf8::upgrade($buf);
+    utf8::encode($buf);
+    $self->add_content($buf);
+}
 
 sub content_ref
 {
@@ -555,7 +562,7 @@ but it will make your program a whole character shorter :-)
 
 =item $mess->content
 
-=item $mess->content( $content )
+=item $mess->content( $bytes )
 
 The content() method sets the raw content if an argument is given.  If no
 argument is given the content is not touched.  In either case the
@@ -565,14 +572,19 @@ Note that the content should be a string of bytes.  Strings in perl
 can contain characters outside the range of a byte.  The C<Encode>
 module can be used to turn such strings into a string of bytes.
 
-=item $mess->add_content( $data )
+=item $mess->add_content( $bytes )
 
-The add_content() methods appends more data to the end of the current
-content buffer.
+The add_content() methods appends more data bytes to the end of the
+current content buffer.
+
+=item $mess->add_content_utf8( $string )
+
+The add_content_utf8() method appends the UTF-8 bytes representing the
+string to the end of the current content buffer.
 
 =item $mess->content_ref
 
-=item $mess->content_ref( \$content )
+=item $mess->content_ref( \$bytes )
 
 The content_ref() method will return a reference to content buffer string.
 It can be more efficient to access the content this way if the content
@@ -591,9 +603,9 @@ add_content() will refuse to do anything.
 
 =item $mess->decoded_content( %options )
 
-Returns the content with any C<Content-Encoding> undone and strings
-mapped to perl's Unicode strings.  If the C<Content-Encoding> or
-C<charset> of the message is unknown this method will fail by
+Returns the content with any C<Content-Encoding> undone and the raw
+content encoded to perl's Unicode strings.  If the C<Content-Encoding>
+or C<charset> of the message is unknown this method will fail by
 returning C<undef>.
 
 The following options can be specified.
@@ -613,7 +625,7 @@ This override the default charset of "ISO-8859-1".
 
 Abort decoding when if malformed characters is found in the content.  By
 default you get the substitution character ("\x{FFFD}") in place of
-mailformed characters.
+malformed characters.
 
 =item C<raise_error>
 
