@@ -1,19 +1,16 @@
-#
-# $Id: file.pm,v 1.15 1996/09/13 15:47:00 aas Exp $
-
 package LWP::Protocol::file;
 
 require LWP::Protocol;
+@ISA = qw(LWP::Protocol);
+
+use strict;
+
 require LWP::MediaTypes;
 require HTTP::Request;
 require HTTP::Response;
 require HTTP::Status;
 require HTTP::Date;
 
-use Carp;
-
-@ISA = qw(LWP::Protocol);
-
 
 sub request
 {
@@ -31,8 +28,7 @@ sub request
     }
 
     # check method
-    $method = $request->method;
-
+    my $method = $request->method;
     unless ($method eq 'GET' || $method eq 'HEAD') {
 	return new HTTP::Response &HTTP::Status::RC_BAD_REQUEST,
 				  'Library does not allow method ' .
@@ -45,17 +41,11 @@ sub request
     my $scheme = $url->scheme;
     if ($scheme ne 'file') {
 	return new HTTP::Response &HTTP::Status::RC_INTERNAL_SERVER_ERROR,
-				  "LWP::file::request called for '$scheme'";
-    }
-
-    my $host = $url->host;
-    if ($host and $host !~ /^localhost$/i) {
-	return new HTTP::Response &HTTP::Status::RC_BAD_REQUEST_CLIENT,
-				  'Only file://localhost/ allowed';
+			   "LWP::Protocol::file::request called for '$scheme'";
     }
 
     # URL OK, look at file
-    my $path  = $url->local_path;
+    my $path  = $url->file;
 
     # test file exists and is readable
     unless (-e $path) {
@@ -85,7 +75,7 @@ sub request
     }
 
     # Ok, should be an OK response by now...
-    $response = new HTTP::Response &HTTP::Status::RC_OK;
+    my $response = new HTTP::Response &HTTP::Status::RC_OK;
 
     # fill in response headers
     $response->header('Last-Modified', HTTP::Date::time2str($mtime));
@@ -99,14 +89,22 @@ sub request
 	closedir(D);
 
 	# Make directory listing
+	require URI::Escape;
+	require HTML::Entities;
+        my $pathe = $path . ( $^O eq 'MacOS' ? ':' : '/');
 	for (@files) {
-	    $_ .= "/" if -d "$path/$_";
-	    $_ = qq{<LI> <a href="$_">$_</a>};
+	    my $furl = URI::Escape::uri_escape($_);
+            if ( -d "$pathe$_" ) {
+                $furl .= '/';
+                $_ .= '/';
+            }
+	    my $desc = HTML::Entities::encode($_);
+	    $_ = qq{<LI><A HREF="$furl">$desc</A>};
 	}
 	# Ensure that the base URL is "/" terminated
 	my $base = $url->clone;
-	unless ($base->epath =~ m|/$|) {
-	    $base->epath($base->epath . "/");
+	unless ($base->path =~ m|/$|) {
+	    $base->path($base->path . "/");
 	}
 	my $html = join("\n",
 			"<HTML>\n<HEAD>",
@@ -119,21 +117,22 @@ sub request
 
 	$response->header('Content-Type',   'text/html');
 	$response->header('Content-Length', length $html);
+	$html = "" if $method eq "HEAD";
 
 	return $self->collect_once($arg, $response, $html);
 
-    } else {            # path is a regular file
-	my($type, @enc) = LWP::MediaTypes::guess_media_type($path);
-	$response->header('Content-Type',   $type) if $type;
-	$response->header('Content-Length', $filesize);
-	for (@enc) {
-	    $response->push_header('Content-Encoding', $_);
-	}
+    }
 
-	# read the file
+    # path is a regular file
+    $response->header('Content-Length', $filesize);
+    LWP::MediaTypes::guess_media_type($path, $response);
+
+    # read the file
+    if ($method ne "HEAD") {
 	open(F, $path) or return new
-	   HTTP::Response(&HTTP::Status::RC_INTERNAL_SERVER_ERROR,
-			  "Cannot read file '$path': $!");
+	    HTTP::Response(&HTTP::Status::RC_INTERNAL_SERVER_ERROR,
+			   "Cannot read file '$path': $!");
+	binmode(F);
 	$response =  $self->collect($arg, $response, sub {
 	    my $content = "";
 	    my $bytes = sysread(F, $content, $size);
