@@ -3,7 +3,7 @@ package Net::HTTP;
 use strict;
 use vars qw($VERSION @ISA);
 
-$VERSION = "5.810";
+$VERSION = "5.811";
 eval { require IO::Socket::INET } || require IO::Socket;
 require Net::HTTP::Methods;
 
@@ -213,7 +213,12 @@ read_response_headers() call.
 
 The return value will be C<undef> on read errors, 0 on EOF, -1 if no data
 could be returned this time, otherwise the number of bytes assigned
-to $buf.  The $buf set to "" when the return value is -1.
+to $buf.  The $buf is set to "" when the return value is -1.
+
+You normally want to retry this call if this function returns either
+-1 or C<undef> with C<$!> as EINTR or EAGAIN (see L<Errno>).  EINTR
+can happen if the application catches signals and EAGAIN can happen if
+you made the socket non-blocking.
 
 This method will raise exceptions (die) if the server does not speak
 proper HTTP.  This can only happen when reading chunked data.
