@@ -5,7 +5,7 @@ require 5.005;  # 4-arg substr
 use strict;
 use vars qw($VERSION);
 
-$VERSION = "5.810";
+$VERSION = "5.811";
 
 my $CRLF = "\015\012";   # "\r\n" is not portable
 
@@ -227,11 +227,26 @@ sub my_readline {
 		if $max_line_length && length($_) > $max_line_length;
 
 	    # need to read more data to find a line ending
-	    my $n = $self->sysread($_, 1024, length);
-	    if (!$n) {
-		return undef unless length;
-		return substr($_, 0, length, "");
-	    }
+          READ:
+            {
+                my $n = $self->sysread($_, 1024, length);
+                unless (defined $n) {
+                    redo READ if $!{EINTR};
+                    if ($!{EAGAIN}) {
+                        # Hmm, we must be reading from a non-blocking socket
+                        # XXX Should really wait until this socket is readable,...
+                        select(undef, undef, undef, 0.1);  # but this will do for now
+                        redo READ;
+                    }
+                    # if we have already accumulated some data let's at least
+                    # return that as a line
+                    die "read failed: $!" unless length;
+                }
+                unless ($n) {
+                    return undef unless length;
+                    return substr($_, 0, length, "");
+                }
+            }
 	}
 	die "Line too long ($pos; limit is $max_line_length)"
 	    if $max_line_length && $pos > $max_line_length;
@@ -366,8 +381,8 @@ sub read_entity_body {
 	delete ${*$self}{'http_bytes'};
 	my $method = shift(@{${*$self}{'http_request_method'}});
 	my $status = ${*$self}{'http_status'};
-	if ($method eq "HEAD" || $status =~ /^(?:1|[23]04)/) {
-	    # these responses are always empty
+	if ($method eq "HEAD") {
+	    # this response is always empty regardless of other headers
 	    $bytes = 0;
 	}
 	elsif (my $te = ${*$self}{'http_te'}) {
@@ -407,6 +422,11 @@ sub read_entity_body {
 	elsif (defined(my $content_length = ${*$self}{'http_content_length'})) {
 	    $bytes = $content_length;
 	}
+        elsif ($status =~ /^(?:1|[23]04)/) {
+            # RFC 2616 says that these responses should always be empty
+            # but that does not appear to be true in practice [RT#17907]
+            $bytes = 0;
+        }
 	else {
 	    # XXX Multi-Part types are self delimiting, but RFC 2616 says we
 	    # only has to deal with 'multipart/byteranges'
